\\ Copyright 2022, 2023 Kevin Ryde

\\ This file is part of pari-primecount.

\\ pari-primecount is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ pari-primecount is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with pari-primecount.  If not, see <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("../primecount.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

my_prime(n) =
{
  primecount_INTERNAL_verbose = n>10^9;
  primecount_nth_prime(n);
\\   if(n < 10^8,
\\      prime(n), 
\\      print("primecount_program_prime() len=",#digits(n),"  ",n);
\\ );
}
check_equal(prime(100), 541, "prime(100)");
check_equal(my_prime(100), 541, "my_prime(100)");


\\-----------------------------------------------------------------------------
\\ A363016  prime index start run of n primes 1 mod 4
\\   bit slow up at n=20

{
  my(want=OEIS_data("A363016"));
  check_equal(#want,33);
  check_equal(#OEIS_data("A057624"),33);
  my(A057624=OEIS_data_func("A057624",['offset,1]));
  my(by_primepi(n) = primecount_pi(A057624(n)));
  check_equal(vector(#want,n,want[n]),
              vector(#want,n,by_primepi(n)),
              "A363016 by_primepi() of A057624");  
}


\\-----------------------------------------------------------------------------
\\ A363017  prime index start run of n primes 3 mod 8


{
  my(want=OEIS_data("A363017"));
  check_equal(#want,20, "A363017 length");
  check_equal(#OEIS_data("A057632"), 20, "A057632 length");
  my(A057632=OEIS_data_func("A057632",['offset,1]));
  my(by_primepi(n) = primecount_pi(A057632(n)));
  check_equal(vector(#want,n,want[n]),
              vector(#want,n,by_primepi(n)),
              "A363017 by_primepi()");  
}


\\-----------------------------------------------------------------------------
\\ A358166  prime steps
{
  \\ code by Michel Marcus in A358166
  my(lista(nn) = my(va = vector(nn));
               va[1] = 13; 
     for (n=2, nn, 
          print("at n="n);
          if (va[n-1] %
               2, va[n] = va[n-1] + my_prime(va[n-1]), va
               [n] = va[n-1]/2); ); va);

  OEIS_check_vecfunc("A358166",lista);

  check_equal(prime(13), 41);
  check_equal(prime(27), 103);

  \\ vector(20,n,if(n%2==0,n/2,n+prime(n)))
  \\ not in OEIS: 3, 1, 8, 2, 16, 3, 24, 4, 32, 5, 42, 6, 54, 7, 62, 8, 76, 9, 86, 10

  \\ quite slow
  if(0,
     my(v=lista(81));
     check_equal(v[81], 1977693361846020549));
}

\\-----------------------------------------------------------------------------
\\ A356082  Matula-Goebel complete binary
\\   OEIS_data("A356082")
\\   top term 
\\   logint(135016053798647886015597889,10) == 26

{
  my(want=OEIS_data("A356082"),
     n=1,
     p=1);
  while(n<=#want,
        check_equal(p, want[n],
                    Str("A356082 complete binary n="n));
        if(p > 10^30, break);
        p = my_prime(p)^2;
        n++);
  print("A356082 check stop at n="n" p="p" len=",#digits(p));
}


\\-----------------------------------------------------------------------------
\\ A352289 caterpillar down

\\ OEIS_data("A352289")
\\ apply(t->#digits(t), OEIS_data("A352289"))
{
  my(want=OEIS_data("A352289"),
     n=1,
     p=1);
  while(1,
        check_equal(p, want[n],
                    Str("A352289 caterpillar n="n));
        if(p > 10^12, break);
        p = 2*my_prime(p);
        n++);
  print("A352289 check stop at n="n" p="p" len=",#digits(p));
}

\\-----------------------------------------------------------------------------
\\ A007097 prime'th recurrence

\\ apply(t->#digits(t), OEIS_data("A007097"))
{
  my(a=OEIS_data_func("A007097"),
     n=0,
     p=1);
  while(1,
        check_equal(p, a(n),
                    Str("A007097 primeth n="n));
        if(p > 10^12, break);
        p = my_prime(p);
        n++);
  print("A007097 check stop at n="n" p="p" len=",#digits(p));
}

\\-----------------------------------------------------------------------------
print("end");
